/*
**	mod_proc.c	- Process related Interface module
**
**
** Copyright (c) 1995-96  Hughes Technologies Pty Ltd
**
** Permission to use, copy, and distribute for non-commercial purposes,
** is hereby granted without fee, providing that the above copyright
** notice appear in all copies and that both the copyright notice and this
** permission notice appear in supporting documentation.
**
** The software may be modified for your own purposes, but modified versions
** may not be distributed.
**
** This software is provided "as is" without any expressed or implied warranty.
**
*/


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>

#include <netdb.h>
#include <netinet/in.h>

#include <ctype.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/wait.h>

#include <common/portability.h>

#include <lite.h>

#include "mod_proc.h"

extern  sym_t   *externReturn;

#ifndef HAVE_SYS_ERRLIST
extern  char    *sys_errlist[];
#endif





/****************************************************************************
** 	_initProcModule
**
**	Purpose	: initialise the proc library
**	Args	: None
**	Returns	: Nothing
**	Notes	: 
*/

void initProcModule()
{
}


void doFork()
{
	int	pid;

	pid = fork();
	if (pid < 0)
		setError(sys_errlist[errno]);
	externReturn = createIntSymbol(pid);
}

void doExec(params)
	plist_t	*params;
{
	char	*path,
		*args[30];
	int	numArgs,
		count;
	sym_t	*cur,
		*array;

	path = params->sym->val;
	array = params->next->sym;
	numArgs = symGetNumArrayElements(array);
	count = 0;
	while(count < numArgs)
	{
		cur = symGetArrayElement(array, count);
		args[count] = cur->val;
		count++;
	}
	args[count] = NULL;
	execv(path, args);
	setError(sys_errlist[errno]);
}


void doPipe()
{
	int	fds[2];
	sym_t	*array;

	if (pipe(fds) < 0)
	{
		setError(sys_errlist[errno]);
		return();
	}
	array = createArray();
	symSetArrayElement(array, 0, createIntSymbol(fds[0]));
	symSetArrayElement(array, 1, createIntSymbol(fds[1]));
	externReturn = array;
}


void doDup2(params)
	plist_t	*params;
{
	int	from,
		to,
		res;

	from = (int) * (int *) params->sym->val;
	to = (int) * (int *) params->next->sym->val;
	res = dup2(from, to);
	if (res < 0)
		setError(sys_errlist[errno]);
	externReturn = createIntSymbol(res);
}


void doWait(params)
	plist_t	*params;
{
	int	pid,
		res;

	pid = (int) * (int *) params->sym->val;
	if (pid == 0)
	{
		res = wait(NULL);
	}
	else
	{
		res = waitpid(pid, NULL, 0);
	}
	if (res < 0)
		setError(sys_errlist[errno]);
	externReturn = createIntSymbol(res);
}


void doSignal(params)
	plist_t	*params;
{
	int	sig;
	char	*funct;

	sig = (int) * (int *)params->sym->val;
	funct = params->next->sym->val;
}


int init_mod(modPtr)
	void	*modPtr;
{
	if (modLoadModuleFunctions(modPtr, proc_efuncts) < 0)
	{
		return(-1);
	}
	else
	{
		return(0);
	}
}
